@@ -27,11 +27,28 @@ and despite the long deprecation, it's in every later release.
 """
 
 import os
+import re
 import sys
 from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 
-from nuitka.utils import Utils
-from nuitka.Version import getNuitkaVersion
+from nuitka.PythonFlavors import (
+    isAnacondaPython,
+    isApplePython,
+    isDebianPackagePython,
+    isMSYS2MingwPython,
+    isNuitkaPython,
+    isPyenvPython,
+    isWinPython,
+)
+from nuitka.utils.FileOperations import getFileContentByLine
+from nuitka.utils.Utils import (
+    getArchitecture,
+    getLinuxDistribution,
+    getOS,
+    isLinux,
+    isPosixWindows,
+)
+from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
 # Indicator if we were called as "nuitka-run" in which case we assume some
 # other defaults and work a bit different with parameters.
@@ -42,17 +59,48 @@ if not is_nuitka_run:
 else:
     usage = "usage: %prog [options] main_module.py"
 
+
+def _getPythonFlavor():
+    # return driven, pylint: disable=too-many-return-statements
+
+    if isNuitkaPython():
+        return "Nuitka Python"
+    elif isAnacondaPython():
+        return "Anaconda Python"
+    elif isWinPython():
+        return "WinPython"
+    elif isDebianPackagePython():
+        return "Debian Python"
+    elif isApplePython():
+        return "Apple Python"
+    elif isPyenvPython():
+        return "pyenv"
+    elif isPosixWindows():
+        return "MSYS2 Posix"
+    elif isMSYS2MingwPython():
+        return "MSYS2 MinGW"
+    else:
+        return "Unknown"
+
+
+def _getVersionInformationValues():
+    # TODO: Might be nice if we could delay version information computation
+    # until it's actually used.
+    yield getNuitkaVersion()
+    yield "Commercial: %s" % getCommercialVersion()
+    yield "Python: %s" % sys.version.split("\n", 1)[0]
+    yield "Flavor: %s" % _getPythonFlavor()
+    yield "Executable: %s" % sys.executable
+    yield "OS: %s" % getOS()
+    yield "Arch: %s" % getArchitecture()
+
+    if isLinux():
+        yield "Distribution: %s %s" % getLinuxDistribution()
+
+
 parser = OptionParser(
     usage=usage,
-    version="\n".join(
-        (
-            getNuitkaVersion(),
-            "Python: " + sys.version.split("\n")[0],
-            "Executable: " + sys.executable,
-            "OS: " + Utils.getOS(),
-            "Arch: " + Utils.getArchitecture(),
-        )
-    ),
+    version="\n".join(_getVersionInformationValues()),
 )
 
 parser.add_option(
@@ -77,15 +125,31 @@ want to use "--python-flag=no_site" to avoid the "site.py" module, which can sav
 a lot of code dependencies. Defaults to off.""",
 )
 
+parser.add_option(
+    "--no-standalone",
+    action="store_false",
+    dest="is_standalone",
+    default=False,
+    help=SUPPRESS_HELP,
+)
+
+
 parser.add_option(
     "--onefile",
     action="store_true",
     dest="is_onefile",
     default=False,
     help="""\
-In case of standalone mode, enable single file mode. This means not a folder,
-but a compressed executable is created and used. Experimental at this time,
-and not supported on all OSes. Defaults to off.""",
+On top of standalone mode, enable onefile mode. This means not a folder,
+but a compressed executable is created and used. Defaults to off.""",
+)
+
+parser.add_option(
+    "--no-onefile",
+    action="store_false",
+    dest="is_onefile",
+    default=False,
+    help=SUPPRESS_HELP,
 )
 
 
@@ -99,7 +163,7 @@ if os.name == "nt":
         help="""\
 Architecture of Python to use. One of "x86" or "x86_64".
 Defaults to what you run Nuitka with (currently "%s")."""
-        % (Utils.getArchitecture()),
+        % (getArchitecture()),
     )
 
 parser.add_option(
@@ -116,19 +180,22 @@ parser.add_option(
     "--python-flag",
     action="append",
     dest="python_flags",
+    metavar="FLAG",
     default=[],
     help="""\
-Python flags to use. Default uses what you are using to run Nuitka, this
+Python flags to use. Default is what you are using to run Nuitka, this
 enforces a specific mode. These are options that also exist to standard
-Python executable. Currently supported: "-S" (alias "nosite"),
+Python executable. Currently supported: "-S" (alias "no_site"),
 "static_hashes" (do not use hash randomization), "no_warnings" (do not
-give Python runtime warnings), "-O" (alias "noasserts"). Default empty.""",
+give Python runtime warnings), "-O" (alias "no_asserts"), "no_docstrings"
+(do not use docstrings). Default empty.""",
 )
 
 parser.add_option(
     "--python-for-scons",
     action="store",
     dest="python_scons",
+    metavar="PATH",
     default=None,
     help="""\
 If using Python3.3 or Python3.4, provide the path of a Python binary to use
@@ -160,7 +227,8 @@ parser.add_option(
     dest="assume_yes_for_downloads",
     default=False,
     help="""\
-Allow Nuitka to download code if necessary, e.g. dependency walker on Windows.""",
+Allow Nuitka to download external code if necessary, e.g. dependency
+walker, ccache, and even gcc on Windows.""",
 )
 
 
@@ -175,10 +243,11 @@ include_group.add_option(
     metavar="PACKAGE",
     default=[],
     help="""\
-Include a whole package. Give as a Python namespace, e.g. ``some_package.sub_package``
+Include a whole package. Give as a Python namespace, e.g. "some_package.sub_package"
 and Nuitka will then find it and include it and all the modules found below that
 disk location in the binary or extension module it creates, and make it available
-for import by the code. Default empty.""",
+for import by the code. To avoid unwanted sub packages, e.g. tests you can e.g. do
+this "--nofollow-import-to=*.tests". Default empty.""",
 )
 
 include_group.add_option(
@@ -188,7 +257,7 @@ include_group.add_option(
     metavar="MODULE",
     default=[],
     help="""\
-Include a single module. Give as a Python namespace, e.g. ``some_package.some_module``
+Include a single module. Give as a Python namespace, e.g. "some_package.some_module"
 and Nuitka will then find it and include it in the binary or extension module
 it creates, and make it available for import by the code. Default empty.""",
 )
@@ -196,19 +265,19 @@ it creates, and make it available for import by the code. Default empty.""",
 include_group.add_option(
     "--include-plugin-directory",
     action="append",
-    dest="recurse_extra",
+    dest="include_extra",
     metavar="MODULE/PACKAGE",
     default=[],
     help="""\
 Include the content of that directory, no matter if it's used by the given main
-program in a visible form. Overrides all other recursion options. Can be given
+program in a visible form. Overrides all other inclusion options. Can be given
 multiple times. Default empty.""",
 )
 
 include_group.add_option(
     "--include-plugin-files",
     action="append",
-    dest="recurse_extra_files",
+    dest="include_extra_files",
     metavar="PATTERN",
     default=[],
     help="""\
@@ -245,7 +314,7 @@ follow_group = OptionGroup(parser, "Control the following into imported modules"
 follow_group.add_option(
     "--follow-stdlib",
     action="store_true",
-    dest="recurse_stdlib",
+    dest="follow_stdlib",
     default=False,
     help="""\
 Also descend into imported modules from standard library. This will increase
@@ -255,17 +324,17 @@ the compilation time by a lot. Defaults to off.""",
 follow_group.add_option(
     "--nofollow-imports",
     action="store_true",
-    dest="recurse_none",
+    dest="follow_none",
     default=False,
     help="""\
-When --recurse-none is used, do not descend into any imported modules at all,
-overrides all other recursion options. Defaults to off.""",
+When --nofollow-imports is used, do not descend into any imported modules at all,
+overrides all other inclusion options. Defaults to off.""",
 )
 
 follow_group.add_option(
     "--follow-imports",
     action="store_true",
-    dest="recurse_all",
+    dest="follow_all",
     default=False,
     help="""\
 When --follow-imports is used, attempt to descend into all imported modules.
@@ -275,7 +344,7 @@ Defaults to off.""",
 follow_group.add_option(
     "--follow-import-to",
     action="append",
-    dest="recurse_modules",
+    dest="follow_modules",
     metavar="MODULE/PACKAGE",
     default=[],
     help="""\
@@ -286,7 +355,7 @@ multiple times. Default empty.""",
 follow_group.add_option(
     "--nofollow-import-to",
     action="append",
-    dest="recurse_not_modules",
+    dest="follow_not_modules",
     metavar="MODULE/PACKAGE",
     default=[],
     help="""\
@@ -304,7 +373,7 @@ data_group.add_option(
     "--include-package-data",
     action="append",
     dest="package_data",
-    metavar="PACKAGE_DATA",
+    metavar="PACKAGE",
     default=[],
     help="""\
 Include data files of the given package name. Can use patterns. By default
@@ -317,16 +386,51 @@ data_group.add_option(
     "--include-data-file",
     action="append",
     dest="data_files",
-    metavar="DATA_FILES",
+    metavar="DESC",
     default=[],
     help="""\
-Include data files by filenames in the distribution. Could use patterns for
-use in glob, if specifying a directory with trailing slash. An example would
-be --include-data-file=/etc/somefile.txt=etc/somefile.txt for plain file copy,
-and you can copy multiple like --include-data-file=/etc/*.txt:etc/ with a
-trailing slash required to use the pattern. Default empty.""",
+Include data files by filenames in the distribution. There are many
+allowed forms. With '--include-data-file=/path/to/file/*.txt=folder_name/some.txt' it
+will copy a single file and complain if it's multiple. With
+'--include-data-file=/path/to/files/*.txt=folder_name/' it will put
+all matching files into that folder. For recursive copy there is a
+form with 3 values that '--include-data-file=/path/to/scan=folder_name=**/*.txt'
+that will preserve directory structure. Default empty.""",
 )
 
+data_group.add_option(
+    "--include-data-dir",
+    action="append",
+    dest="data_dirs",
+    metavar="DIRECTORY",
+    default=[],
+    help="""\
+Include data files from complete directory in the distribution. This is
+recursive. Check '--include-data-file' with patterns if you want non-recursive
+inclusion. An example would be '--include-data-dir=/path/somedir=data/somedir'
+for plain copy, of the whole directory. All files are copied, if you want to
+exclude files you need to remove them beforehand. Default empty.""",
+)
+
+data_files_tags = [("inhibit", "do not include the file")]
+
+# TODO: Expose this when finished, pylint: disable=using-constant-test
+if False:
+    data_group.add_option(
+        "--data-file-tags",
+        action="append",
+        dest="data_tags",
+        metavar="DATA_TAGS",
+        default=[],
+        help="""\
+    For included data files, special handlings can be chosen. With the
+    commercial plugins, e.g. files can be included directly in the
+    binary. The list is completed by some plugins. With the current
+    list of plugins, these are available: %s.
+    The default is empty."""
+        % ",".join("'%s' (%s)" % d for d in data_files_tags),
+    )
+
 
 parser.add_option_group(data_group)
 
@@ -401,6 +505,7 @@ codegen_group.add_option(
     "--file-reference-choice",
     action="store",
     dest="file_reference_mode",
+    metavar="MODE",
     choices=("original", "runtime", "frozen"),
     default=None,
     help="""\
@@ -428,11 +533,11 @@ output_group.add_option(
     help="""\
 Specify how the executable should be named. For extension modules there is no
 choice, also not for standalone mode and using it will be an error. This may
-include path information that needs to exist though. Defaults to %s on this
+include path information that needs to exist though. Defaults to '%s' on this
 platform.
 """
     % "<program_name>"
-    + (".exe" if Utils.getOS() == "Windows" else ".bin"),
+    + (".exe" if getOS() == "Windows" else ".bin"),
 )
 
 output_group.add_option(
@@ -550,6 +655,7 @@ debug_group.add_option(
     "--experimental",
     action="append",
     dest="experimental",
+    metavar="FLAG",
     default=[],
     help="""\
 Use features declared as 'experimental'. May have no effect if no experimental
@@ -565,6 +671,17 @@ debug_group.add_option(
     help=SUPPRESS_HELP,
 )
 
+debug_group.add_option(
+    "--low-memory",
+    action="store_true",
+    dest="low_memory",
+    default=False,
+    help="""\
+Attempt to use less memory, by forking less C compilation jobs and using
+options that use less memory. For use on embedded machines. Use this in
+case of out of memory problems. Defaults to off.""",
+)
+
 if os.name == "nt":
     debug_group.add_option(
         "--disable-dll-dependency-cache",
@@ -616,28 +733,28 @@ Enforce the use of clang. On Windows this requires a working Visual
 Studio version to piggy back on. Defaults to off.""",
 )
 
-if os.name == "nt":
-    c_compiler_group.add_option(
-        "--mingw64",
-        action="store_true",
-        dest="mingw64",
-        default=False,
-        help="""\
+c_compiler_group.add_option(
+    "--mingw64",
+    action="store_true",
+    dest="mingw64",
+    default=False,
+    help="""\
 Enforce the use of MinGW64 on Windows. Defaults to off.""",
-    )
+)
 
-    c_compiler_group.add_option(
-        "--msvc",
-        action="store",
-        dest="msvc",
-        default=None,
-        help="""\
+c_compiler_group.add_option(
+    "--msvc",
+    action="store",
+    dest="msvc_version",
+    default=None,
+    help="""\
 Enforce the use of specific MSVC version on Windows. Allowed values
-are e.g. 14.0, specify an illegal value for a list of installed compilers,
-beware that only latest MSVC is really supported.
+are e.g. "14.2" (MSVC 2019), specify an illegal value for a list of
+installed compilers, or use "latest". Notice that only latest
+MSVC is really supported, and you can use "latest" to enforce that.
 
-Defaults to the most recent version.""",
-    )
+Defaults to MSVC on Windows being used if installed, otherwise MinGW64.""",
+)
 
 c_compiler_group.add_option(
     "-j",
@@ -645,7 +762,7 @@ c_compiler_group.add_option(
     action="store",
     dest="jobs",
     metavar="N",
-    default=Utils.getCoreCount(),
+    default=None,
     help="""\
 Specify the allowed number of parallel C compiler jobs. Defaults to the
 system CPU count.""",
@@ -653,16 +770,102 @@ system CPU count.""",
 
 c_compiler_group.add_option(
     "--lto",
-    action="store_true",
+    action="store",
     dest="lto",
+    metavar="choice",
+    default="auto",
+    choices=("yes", "no", "auto"),
+    help="""\
+Use link time optimizations (MSVC, gcc, clang). Allowed values are
+"yes", "no", and "auto" (when it's known to work). Defaults to
+"auto".""",
+)
+
+c_compiler_group.add_option(
+    "--static-libpython",
+    action="store",
+    dest="static_libpython",
+    metavar="choice",
+    default="auto",
+    choices=("yes", "no", "auto"),
+    help="""\
+Use static link library of Python. Allowed values are "yes", "no",
+and "auto" (when it's known to work). Defaults to "auto".""",
+)
+
+c_compiler_group.add_option(
+    "--disable-ccache",
+    action="store_true",
+    dest="disable_ccache",
     default=False,
     help="""\
-Use link time optimizations if available and usable (MSVC or gcc 4.6 and higher).
-Defaults to off.""",
+Do not attempt to use ccache (gcc, clang, etc.) or clcache (MSVC, clangcl).""",
 )
 
 parser.add_option_group(c_compiler_group)
 
+pgo_group = OptionGroup(parser, "PGO compilation choices")
+
+pgo_group.add_option(
+    "--pgo",
+    action="store_true",
+    dest="is_c_pgo",
+    default=False,
+    help="""\
+Enables C level profile guided optimization (PGO), by executing a dedicated build first
+for a profiling run, and then using the result to feedback into the C compilation.
+Note: This is experimental and not working with standalone modes of Nuitka yet.
+Defaults to off.""",
+)
+
+pgo_group.add_option(
+    "--pgo-python",
+    action="store_true",
+    dest="is_python_pgo",
+    default=False,
+    help=SUPPRESS_HELP,
+)
+
+pgo_group.add_option(
+    "--pgo-python-input",
+    action="store",
+    dest="python_pgo_input",
+    default=None,
+    help=SUPPRESS_HELP,
+)
+
+pgo_group.add_option(
+    "--pgo-python-policy-unused-module",
+    action="store",
+    dest="python_pgo_policy_unused_module",
+    choices=("include", "exclude", "bytecode"),
+    default="include",
+    help=SUPPRESS_HELP,
+)
+
+pgo_group.add_option(
+    "--pgo-args",
+    action="store",
+    dest="pgo_args",
+    default="",
+    help="""\
+Arguments to be passed in case of profile guided optimization. These are passed to the special
+built executable during the PGO profiling run. Default empty.""",
+)
+
+pgo_group.add_option(
+    "--pgo-executable",
+    action="store",
+    dest="pgo_executable",
+    default=None,
+    help="""\
+Command to execute when collecting profile information. Use this only, if you need to
+launch it through a script that prepares it to run. Default use created program.""",
+)
+
+
+parser.add_option_group(pgo_group)
+
 tracing_group = OptionGroup(parser, "Tracing features")
 
 tracing_group.add_option(
@@ -695,7 +898,7 @@ Defaults to off.""",
 )
 
 tracing_group.add_option(
-    "--no-progress",
+    "--no-progressbar",
     action="store_false",
     dest="progress_bar",
     default=True,
@@ -728,6 +931,7 @@ tracing_group.add_option(
     "--show-modules-output",
     action="store",
     dest="show_inclusion_output",
+    metavar="PATH",
     default=None,
     help="""\
 Where to output --show-modules, should be a filename. Default is standard output.""",
@@ -747,6 +951,7 @@ tracing_group.add_option(
     "--verbose-output",
     action="store",
     dest="verbose_output",
+    metavar="PATH",
     default=None,
     help="""\
 Where to output --verbose, should be a filename. Default is standard output.""",
@@ -767,7 +972,7 @@ windows_group.add_option(
 windows_group.add_option(
     "--windows-disable-console",
     action="store_true",
-    dest="win_disable_console",
+    dest="disable_console",
     default=False,
     help="""\
 When compiling for Windows, disable the console window. Defaults to off.""",
@@ -795,6 +1000,15 @@ windows_group.add_option(
     help="Copy executable icons from this existing executable (Windows only).",
 )
 
+windows_group.add_option(
+    "--onefile-windows-splash-screen-image",
+    action="store",
+    dest="splash_screen_image",
+    default=None,
+    help="""\
+When compiling for Windows and onefile, show this while loading the application. Defaults to off.""",
+)
+
 windows_group.add_option(
     "--windows-uac-admin",
     action="store_true",
@@ -848,7 +1062,7 @@ windows_group.add_option(
     default=None,
     help="""\
 File version to use in Windows Version information. Must be a sequence of
-up to 4 numbers, nothing else allowed.
+up to 4 numbers, e.g. 1.0.0.0, only this format is allowed.
 One of file or product version is required, when a version resource needs to be
 added, e.g. to specify product name, or company name. Defaults to unused.""",
 )
@@ -861,7 +1075,7 @@ windows_group.add_option(
     default=None,
     help="""\
 Product version to use in Windows Version information. Must be a sequence of
-up to 4 numbers, nothing else allowed.
+up to 4 numbers, e.g. 1.0.0.0, only this format is allowed.
 One of file or product version is required, when a version resource needs to be
 added, e.g. to specify product name, or company name. Defaults to unused.""",
 )
@@ -881,19 +1095,19 @@ added, e.g. to specify product name, or company name. Defaults to nonsense.""",
 
 windows_group.add_option(
     "--windows-onefile-tempdir",
+    "--onefile-tempdir",
     action="store_true",
-    dest="is_windows_onefile_tempdir",
-    metavar="WINDOWS_ONEFILE_TEMPDIR",
+    dest="is_onefile_tempdir",
+    metavar="ONEFILE_TEMPDIR",
     default=False,
-    help="""\
-Use temporary folder rather than company AppData. Defaults to off.""",
+    help=SUPPRESS_HELP,
 )
 
 windows_group.add_option(
     "--windows-onefile-tempdir-spec",
     action="store",
-    dest="windows_onefile_tempdir_spec",
-    metavar="WINDOWS_ONEFILE_TEMPDIR_SPEC",
+    dest="onefile_tempdir_spec",
+    metavar="ONEFILE_TEMPDIR_SPEC",
     default=None,
     help="""\
 Use this as a temporary folder. Defaults to '%TEMP%\\onefile_%PID%_%TIME%', i.e. system temporary directory.""",
@@ -926,6 +1140,75 @@ to not active, use e.g. '%PROGRAM%.err.txt', i.e. file near your program.""",
 
 parser.add_option_group(windows_group)
 
+macos_group = OptionGroup(parser, "macOS specific controls")
+
+macos_group.add_option(
+    "--macos-onefile-icon",
+    action="append",
+    dest="icon_path",
+    metavar="ICON_PATH",
+    default=[],
+    help="Add executable icon for binary to use. Can be given only one time. Defaults to Python icon if available.",
+)
+
+macos_group.add_option(
+    "--macos-disable-console",
+    action="store_true",
+    dest="disable_console",
+    default=False,
+    help="""\
+When compiling for macOS, disable the console window and create a GUI
+application. Defaults to off.""",
+)
+
+macos_group.add_option(
+    "--macos-create-app-bundle",
+    action="store_true",
+    dest="macos_create_bundle",
+    default=False,
+    help="""\
+When compiling for macOS, create a bundle rather than a plain binary
+application. Currently experimental and incomplete. Currently this
+is the only way to unlock disabling of console.Defaults to off.""",
+)
+
+macos_group.add_option(
+    "--macos-signed-app-name",
+    action="store",
+    dest="macos_signed_app_name",
+    metavar="MACOS_SIGNED_APP_NAME",
+    default=None,
+    help="""\
+Name of the application to use for macOS signing. Follow com.yourcompany.appname naming
+results for best results, as these have to be globally unique, and will grant protected
+API accesses.""",
+)
+
+macos_group.add_option(
+    "--macos-app-name",
+    action="store",
+    dest="macos_app_name",
+    metavar="MACOS_APP_NAME",
+    default=None,
+    help="""\
+Name of the product to use in macOS bundle information. Defaults to base
+filename of the binary.""",
+)
+
+macos_group.add_option(
+    "--macos-app-version",
+    action="store",
+    dest="macos_app_version",
+    metavar="MACOS_APP_VERSION",
+    default=None,
+    help="""\
+Product version to use in macOS bundle information. Defaults to 1.0 if
+not given.""",
+)
+
+
+parser.add_option_group(macos_group)
+
 linux_group = OptionGroup(parser, "Linux specific controls")
 
 linux_group.add_option(
@@ -934,7 +1217,7 @@ linux_group.add_option(
     dest="icon_path",
     metavar="ICON_PATH",
     default=[],
-    help="Add executable icon for onefile binary to use. Can be given only one time. Defaults to ",
+    help="Add executable icon for onefile binary to use. Can be given only one time. Defaults to Python icon if available.",
 )
 
 parser.add_option_group(linux_group)
@@ -942,10 +1225,11 @@ parser.add_option_group(linux_group)
 plugin_group = OptionGroup(parser, "Plugin control")
 
 plugin_group.add_option(
-    "--plugin-enable",
     "--enable-plugin",
+    "--plugin-enable",
     action="append",
     dest="plugins_enabled",
+    metavar="PLUGIN_NAME",
     default=[],
     help="""\
 Enabled plugins. Must be plug-in names. Use --plugin-list to query the
@@ -953,10 +1237,11 @@ full list and exit. Default empty.""",
 )
 
 plugin_group.add_option(
-    "--plugin-disable",
     "--disable-plugin",
+    "--plugin-disable",
     action="append",
     dest="plugins_disabled",
+    metavar="PLUGIN_NAME",
     default=[],
     help="""\
 Disabled plugins. Must be plug-in names. Use --plugin-list to query the
@@ -970,9 +1255,9 @@ plugin_group.add_option(
     default=True,
     help="""\
 Plugins can detect if they might be used, and the you can disable the warning
-via --plugin-disable=plugin-that-warned, or you can use this option to disable
+via "--disable-plugin=plugin-that-warned", or you can use this option to disable
 the mechanism entirely, which also speeds up compilation slightly of course as
-this detection code is run in vain once you are certain of which plug-ins to
+this detection code is run in vain once you are certain of which plugins to
 use. Defaults to off.""",
 )
 
@@ -992,13 +1277,26 @@ plugin_group.add_option(
     "--user-plugin",
     action="append",
     dest="user_plugins",
+    metavar="PATH",
     default=[],
     help="The file name of user plugin. Can be given multiple times. Default empty.",
 )
 
+plugin_group.add_option(
+    "--persist-source-changes",
+    action="store_true",
+    dest="persist_source_changes",
+    default=False,
+    help="""\
+Write source changes to original Python files. Use with care. May need
+permissions, best for use in a virtualenv to debug if plugin code
+changes work with standard Python or to benefit from bloat removal
+even with pure Python. Default False.""",
+)
+
 
 def _considerPluginOptions(logger):
-    # Recursive dependency on plugins during parsing of command line.
+    # Cyclic dependency on plugins during parsing of command line.
     from nuitka.plugins.Plugins import (
         addPluginCommandLineOptions,
         addUserPluginCommandLineOptions,
@@ -1009,12 +1307,14 @@ def _considerPluginOptions(logger):
             plugin_names = arg[16:]
             if "=" in plugin_names:
                 logger.sysexit(
-                    "Error, plugin options format changed. Use '--plugin-enable=%s --help' to know new options."
+                    "Error, plugin options format changed. Use '--enable-plugin=%s --help' to know new options."
                     % plugin_names.split("=", 1)[0]
                 )
 
             addPluginCommandLineOptions(
-                parser=parser, plugin_names=plugin_names.split(",")
+                parser=parser,
+                plugin_names=plugin_names.split(","),
+                data_files_tags=data_files_tags,
             )
 
         if arg.startswith("--user-plugin="):
@@ -1022,13 +1322,135 @@ def _considerPluginOptions(logger):
             if "=" in plugin_name:
                 logger.sysexit(
                     "Error, plugin options format changed. Use '--user-plugin=%s --help' to know new options."
-                    % plugin_name
+                    % plugin_name.split("=", 1)[0]
                 )
 
-            addUserPluginCommandLineOptions(parser=parser, filename=plugin_name)
+            addUserPluginCommandLineOptions(
+                parser=parser, filename=plugin_name, data_files_tags=data_files_tags
+            )
+
+
+def _expandProjectArg(arg, filename_arg, for_eval):
+    def wrap(value):
+        if for_eval:
+            return repr(value)
+        else:
+            return value
+
+    values = {
+        "OS": wrap(getOS()),
+        "Arch": wrap(getArchitecture()),
+        "Flavor": wrap(_getPythonFlavor()),
+        "Version": getNuitkaVersion(),
+        "Commercial": wrap(getCommercialVersion()),
+        "MAIN_DIRECTORY": wrap(os.path.dirname(filename_arg) or "."),
+    }
+
+    if getOS() != "Linux":
+        dist_info = "N/A", "0"
+    else:
+        dist_info = getLinuxDistribution()
+
+    values["Linux_Distribution_Name"] = dist_info[0]
+    values["Linux_Distribution_Version"] = dist_info[1]
+
+    arg = arg.format(**values)
+
+    return arg
+
+
+def _getProjectOptions(logger, filename_arg, module_mode):
+    # Complex stuff, pylint: disable=too-many-branches,too-many-locals
+    # Do it only once.
+    if os.environ.get("NUITKA_REEXECUTION", "0") == "1":
+        return
+
+    if os.path.isdir(filename_arg):
+        if module_mode:
+            filename_arg = os.path.join(filename_arg, "__init__.py")
+        else:
+            filename_arg = os.path.join(filename_arg, "__main__.py")
+
+    # The file specified may not exist, let the later parts of Nuitka handle this.
+    try:
+        contents_by_line = getFileContentByLine(filename_arg, "rb")
+    except (OSError, IOError):
+        return
+
+    def sysexit(count, message):
+        logger.sysexit("%s:%d %s" % (filename_arg, count + 1, message))
+
+    execute_block = True
+    expect_block = False
+
+    cond_level = -1
+
+    for count, line in enumerate(contents_by_line):
+        match = re.match(b"^\\s*#(\\s+)nuitka-project(.*?):(.*)", line)
+
+        if match:
+            level, command, arg = match.groups()
+            level = len(level)
+            arg = arg.rstrip()
+
+            # Check for empty conditional blocks.
+            if expect_block and level <= cond_level:
+                sysexit(
+                    count,
+                    "Error, 'nuitka-project-if' is expected to be followed by block start.",
+                )
+
+            expect_block = False
+
+            if level <= cond_level:
+                execute_block = True
+
+            if level > cond_level and not execute_block:
+                continue
+
+            if str is not bytes:
+                command = command.decode("utf8")
+                arg = arg.decode("utf8")
+
+            if command == "-if":
+                if not arg.endswith(":"):
+                    sysexit(
+                        count,
+                        "Error, 'nuitka-project-if' needs to start a block with a colon at line end.",
+                    )
+
+                arg = arg[:-1]
+
+                expanded = _expandProjectArg(arg, filename_arg, for_eval=True)
+                r = eval(  # We allow the user to run any code, pylint: disable=eval-used
+                    expanded
+                )
+
+                # Likely mistakes, e.g. with "in" tests.
+                if r is not True and r is not False:
+                    sys.exit(
+                        "Error, 'nuitka-project-if' condition %r (%r) does not yield boolean result %r"
+                        % (arg, expanded, r)
+                    )
+
+                execute_block = r
+                expect_block = True
+                cond_level = level
+            elif command == "":
+                arg = re.sub(r"""^([\w-]*=)(['"])(.*)\2$""", r"\1\3", arg.lstrip())
+
+                if not arg:
+                    continue
+
+                yield _expandProjectArg(arg, filename_arg, for_eval=False)
+            else:
+                assert False, (command, line)
 
 
 def parseOptions(logger):
+    # Pretty complex code, having a small options parser and many details as
+    # well as integrating with plugins and run modes, pylint: disable=too-many-branches
+
     # First, isolate the first non-option arguments.
     extra_args = []
 
@@ -1040,6 +1462,7 @@ def parseOptions(logger):
                 continue
 
             if arg[0] != "-":
+                filename_arg = arg[0]
                 break
 
             # Treat "--" as a terminator.
@@ -1051,6 +1474,23 @@ def parseOptions(logger):
             extra_args = sys.argv[count + 1 :]
             sys.argv = sys.argv[0 : count + 1]
 
+    filename_arg = None
+
+    for count, arg in enumerate(sys.argv):
+        if count == 0:
+            continue
+
+        if arg[0] != "-":
+            filename_arg = arg
+            break
+
+    if filename_arg is not None:
+        sys.argv = (
+            [sys.argv[0]]
+            + list(_getProjectOptions(logger, filename_arg, "--module" in sys.argv[1:]))
+            + sys.argv[1:]
+        )
+
     # Next, lets activate plugins early, so they can inject more options to the parser.
     _considerPluginOptions(logger)
 
