@@ -15,30 +15,47 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Options module """
+""" Options module
+
+This exposes the choices made by the user. Defaults will be applied here, and
+some handling of defaults.
+"""
 
 import os
+import shlex
 import sys
 
 from nuitka import Progress, Tracing
 from nuitka.containers.oset import OrderedSet
-from nuitka.OptionParsing import parseOptions
+from nuitka.OptionParsing import isPyenvPython, parseOptions
+from nuitka.PythonFlavors import (
+    isAnacondaPython,
+    isDebianPackagePython,
+    isNuitkaPython,
+    isUninstalledPython,
+)
 from nuitka.PythonVersions import (
     getSupportedPythonVersions,
-    getSystemStaticLibPythonPath,
-    isNuitkaPython,
-    isStaticallyLinkedPython,
-    isUninstalledPython,
+    python_version,
     python_version_str,
 )
+from nuitka.utils.Execution import getExecutablePath
 from nuitka.utils.FileOperations import (
+    isPathExecutable,
     openTextFile,
     resolveShellPatternToFilenames,
 )
+from nuitka.utils.StaticLibraries import getSystemStaticLibPythonPath
 from nuitka.utils.Utils import (
     getArchitecture,
+    getCoreCount,
     getOS,
     hasOnefileSupportedOS,
+    hasStandaloneSupportedOS,
+    isFreeBSD,
+    isLinux,
+    isMacOS,
+    isOpenBSD,
     isWin32Windows,
 )
 
@@ -49,17 +66,22 @@ is_nuitka_run = None
 is_debug = None
 is_nondebug = None
 is_fullcompat = None
+is_report_missing = None
 
 
 def parseArgs(will_reexec):
+    """Parse the command line arguments, with the knowledge if Nuitka will re-execute itself.
+
+    :meta private:
+    """
     # singleton with many cases checking the options right away.
     # pylint: disable=global-statement,too-many-branches,too-many-locals,too-many-statements
-    global is_nuitka_run, options, positional_args, extra_args, is_debug, is_nondebug, is_fullcompat
+    global is_nuitka_run, options, positional_args, extra_args, is_debug, is_nondebug, is_fullcompat, is_report_missing
 
     if os.name == "nt":
         # Windows store Python's don't allow looking at the python, catch that.
         try:
-            with open(sys.executable):
+            with openTextFile(sys.executable, "rb"):
                 pass
         except OSError:
             Tracing.general.sysexit(
@@ -70,6 +92,13 @@ def parseArgs(will_reexec):
         logger=Tracing.options_logger
     )
 
+    is_debug = _isDebug()
+    is_nondebug = not is_debug
+    is_fullcompat = _isFullCompat()
+
+    # TODO: Have dedicated option for it.
+    is_report_missing = is_debug
+
     if options.quiet or int(os.environ.get("NUITKA_QUIET", "0")):
         Tracing.setQuiet()
 
@@ -136,7 +165,7 @@ makes no sense to include a Python runtime."""
         if bad:
             Tracing.options_logger.sysexit(
                 """\
-Error, '--follow-import-to' takes only module names, not directory path '%s'."""
+Error, '--follow-import-to' takes only module names or patterns, not directory path '%s'."""
                 % any_case_module
             )
 
@@ -154,7 +183,7 @@ Error, '--follow-import-to' takes only module names, not directory path '%s'."""
         if bad:
             Tracing.options_logger.sysexit(
                 """\
-Error, '--nofollow-import-to' takes only module names, not directory path '%s'."""
+Error, '--nofollow-import-to' takes only module names or patterns, not directory path '%s'."""
                 % no_case_module
             )
 
@@ -175,7 +204,7 @@ but not for module mode where filenames are mandatory, and not for
 standalone where there is a sane default used inside the dist folder."""
         )
 
-    if getOS() == "Linux":
+    if isLinux():
         if len(getIconPaths()) > 1:
             Tracing.options_logger.sysexit("Error, can only use one icon on Linux.")
 
@@ -229,6 +258,15 @@ standalone where there is a sane default used inside the dist folder."""
             """Error, empty string is not an acceptable product name."""
         )
 
+    splash_screen_filename = getWindowsSplashScreen()
+
+    if splash_screen_filename is not None:
+        if not os.path.isfile(splash_screen_filename):
+            Tracing.options_logger.sysexit(
+                "Error, specified splash screen image '%s' does not exist."
+                % splash_screen_filename
+            )
+
     if file_version or product_version or getWindowsVersionInfoStrings():
         if not (file_version or product_version) and getWindowsCompanyName():
             Tracing.options_logger.sysexit(
@@ -346,16 +384,44 @@ standalone where there is a sane default used inside the dist folder."""
 
     if options.static_libpython == "yes" and getSystemStaticLibPythonPath() is None:
         Tracing.options_logger.sysexit(
-            "Error, static libpython is not found for this Python installation."
+            "Error, static libpython is not found or not supported for this Python installation."
         )
 
-    is_debug = _isDebug()
-    is_nondebug = not is_debug
-    is_fullcompat = _isFullCompat()
+    if (
+        not will_reexec
+        and shallUseStaticLibPython()
+        and getSystemStaticLibPythonPath() is None
+    ):
+        Tracing.options_logger.sysexit(
+            """Error, usable static libpython is not found for this Python installation. You \
+might be missing required '-dev' packages. Disable with --static-libpython=no" if you don't \
+want to install it."""
+        )
+
+    if isStandaloneMode() and isMacOS() and sys.executable.startswith("/usr/bin/"):
+        Tracing.options_logger.sysexit(
+            "Error, Apple Python from macOS is not supported, use e.g. CPython instead."
+        )
+
+    if isStandaloneMode() and isLinux() and getExecutablePath("patchelf") is None:
+        Tracing.options_logger.sysexit(
+            "Error, standalone mode on Linux requires 'patchelf' to be installed. Use 'apt/dnf/yum install patchelf' first."
+        )
+
+    pgo_executable = getPgoExecutable()
+    if pgo_executable and not isPathExecutable(pgo_executable):
+        Tracing.options_logger.sysexit(
+            "Error, path '%s' to binary to use for PGO is not executable."
+            % pgo_executable
+        )
 
 
 def commentArgs():
-    """Comment on options, where we know something is not having the intended effect."""
+    """Comment on options, where we know something is not having the intended effect.
+
+    :meta private:
+
+    """
     # A ton of cases to consider, pylint: disable=too-many-branches
 
     # Inform the user about potential issues with the running version. e.g. unsupported
@@ -378,16 +444,18 @@ def commentArgs():
     else:
         if options.file_reference_mode != default_reference_mode:
             Tracing.options_logger.warning(
-                "Using non-default file reference mode %r rather than %r may cause runtime issues."
+                "Using non-default file reference mode '%s' rather than '%s' may cause runtime issues."
                 % (getFileReferenceMode(), default_reference_mode)
             )
         else:
             Tracing.options_logger.info(
-                "Using default file reference mode %r need not be specified."
+                "Using default file reference mode '%s' need not be specified."
                 % default_reference_mode
             )
 
+    # TODO: Not all of these are usable with MSYS2 really, split those off.
     if getOS() != "Windows":
+        # Too many Windows specific options clearly, pylint: disable=too-many-boolean-expressions
         if (
             getWindowsIconExecutablePath()
             or shallAskForWindowsAdminRights()
@@ -398,11 +466,18 @@ def commentArgs():
             or getWindowsFileVersion()
             or getForcedStderrPath()  # not yet for other platforms
             or getForcedStdoutPath()
+            or getWindowsSplashScreen()
+            or getIntendedPythonArch()
         ):
             Tracing.options_logger.warning(
                 "Using Windows specific options has no effect on other platforms."
             )
 
+        if options.mingw64 or options.msvc_version:
+            Tracing.options_logger.warning(
+                "Requesting Windows specific compilers has no effect on other platforms."
+            )
+
     if isOnefileMode():
         standalone_mode = "onefile"
     elif isStandaloneMode():
@@ -410,24 +485,23 @@ def commentArgs():
     else:
         standalone_mode = None
 
-    if standalone_mode and getOS() == "NetBSD":
+    if standalone_mode and not hasStandaloneSupportedOS():
         Tracing.options_logger.warning(
-            "Standalone mode on NetBSD is not functional, due to $ORIGIN linkage not being supported."
+            "Standalone mode on %s is not known to be supported, might fail to work."
+            % getOS()
         )
 
     if options.follow_all and standalone_mode:
-        if standalone_mode:
-            Tracing.options_logger.info(
-                "Following all imports is the default for %s mode and need not be specified."
-                % standalone_mode
-            )
+        Tracing.options_logger.info(
+            "Following all imports is the default for %s mode and need not be specified."
+            % standalone_mode
+        )
 
     if options.follow_none and standalone_mode:
-        if standalone_mode:
-            Tracing.options_logger.warning(
-                "Following no imports is unlikely to work for %s mode and should not be specified."
-                % standalone_mode
-            )
+        Tracing.options_logger.warning(
+            "Following no imports is unlikely to work for %s mode and should not be specified."
+            % standalone_mode
+        )
 
     if options.dependency_tool:
         Tracing.options_logger.warning(
@@ -436,47 +510,92 @@ def commentArgs():
 
     if shallMakeModule() and options.static_libpython == "yes":
         Tracing.options_logger.warning(
-            "In module mode, providing --static-libpython has no effect, it's not used."
+            "In module mode, providing '--static-libpython' has no effect, it's not used."
         )
 
+        options.static_libpython = "no"
+
+    if (
+        not isPgoMode()
+        and not isPythonPgoMode()
+        and (getPgoArgs() or getPgoExecutable())
+    ):
+        Tracing.optimization_logger.warning(
+            "Providing PGO arguments without enabling PGO mode has no effect."
+        )
+
+    if isPgoMode():
+        if isStandaloneMode():
+            Tracing.optimization_logger.warning(
+                "Using PGO with standalone/onefile mode is not currently working. Expect errors."
+            )
+
+        if shallMakeModule():
+            Tracing.optimization_logger.warning(
+                "Using PGO with module mode is not currently working. Expect errors."
+            )
+
+    if (
+        options.static_libpython == "auto"
+        and not shallMakeModule()
+        and not shallDumpBuiltTreeXML()
+        and not shallUseStaticLibPython()
+        and getSystemStaticLibPythonPath() is not None
+    ):
+        Tracing.options_logger.info(
+            """Detected static libpython to exist, consider '--static-libpython=yes' for better performance, \
+but errors may happen."""
+        )
+
+    if not shallExecuteImmediately():
+        if shallRunInDebugger():
+            Tracing.options_logger.warning(
+                "The '--debugger' option has no effect outside of '--debug' without '--run' option."
+            )
+
+        if not shallClearPythonPathEnvironment():
+            Tracing.options_logger.warning(
+                "The '--execute-with-pythonpath' option has no effect without '--run' option."
+            )
+
 
 def isVerbose():
-    """*bool* = "--verbose" """
+    """:returns: bool derived from ``--verbose``"""
     return options is not None and options.verbose
 
 
 def shallTraceExecution():
-    """*bool* = "--trace-execution" """
+    """:returns: bool derived from ``--trace-execution``"""
     return options.trace_execution
 
 
 def shallExecuteImmediately():
-    """*bool* = "--run" """
+    """:returns: bool derived from ``--run``"""
     return options.immediate_execution
 
 
 def shallRunInDebugger():
-    """*bool* = "--debug" """
+    """:returns: bool derived from ``--debug``"""
     return options.debugger
 
 
 def shallDumpBuiltTreeXML():
-    """*bool* = "--xml" """
+    """:returns: bool derived from ``--xml``"""
     return options.dump_xml
 
 
 def shallOnlyExecCCompilerCall():
-    """*bool* = "--recompile-c-only" """
+    """:returns: bool derived from ``--recompile-c-only``"""
     return options.recompile_c_only
 
 
 def shallNotDoExecCCompilerCall():
-    """*bool* = "--generate-c-only" """
+    """:returns: bool derived from ``--generate-c-only``"""
     return options.generate_c_only
 
 
 def getFileReferenceMode():
-    """*str*, one of "runtime", "original", "frozen", coming from "--file-reference-choice"
+    """*str*, one of "runtime", "original", "frozen", coming from ``--file-reference-choice``
 
     Notes:
         Defaults to runtime for modules and packages, as well as standalone binaries,
@@ -487,32 +606,32 @@ def getFileReferenceMode():
 
 
 def shallMakeModule():
-    """*bool* = "--module" """
+    """:returns: bool derived from ``--module``"""
     return not options.executable
 
 
 def shallCreatePyiFile():
-    """*bool* = **not** "--no-pyi-file" """
+    """*bool* = **not** ``--no-pyi-file``"""
     return options.pyi_file
 
 
 def isAllowedToReexecute():
-    """*bool* = **not** "--must-not-re-execute" """
+    """*bool* = **not** ``--must-not-re-execute``"""
     return options.allow_reexecute
 
 
 def shallFollowStandardLibrary():
-    """*bool* = "--follow-stdlib" """
-    return options.recurse_stdlib
+    """:returns: bool derived from ``--follow-stdlib``"""
+    return options.follow_stdlib
 
 
 def shallFollowNoImports():
-    """*bool* = "--nofollow-imports" """
+    """:returns: bool derived from ``--nofollow-imports``"""
     return options.follow_none
 
 
 def shallFollowAllImports():
-    """*bool* = "--follow-imports" """
+    """:returns: bool derived from ``--follow-imports``"""
     return options.is_standalone or options.follow_all
 
 
@@ -521,12 +640,12 @@ def _splitShellPattern(value):
 
 
 def getShallFollowInNoCase():
-    """*list*, items of "--nofollow-import-to=" """
+    """*list*, items of ``--nofollow-import-to=``"""
     return sum([_splitShellPattern(x) for x in options.follow_not_modules], [])
 
 
 def getShallFollowModules():
-    """*list*, items of "--follow-import-to=" """
+    """*list*, items of ``--follow-import-to=``"""
     return sum(
         [
             _splitShellPattern(x)
@@ -539,32 +658,32 @@ def getShallFollowModules():
 
 
 def getShallFollowExtra():
-    """*list*, items of "--include-plugin-directory=" """
-    return sum([_splitShellPattern(x) for x in options.recurse_extra], [])
+    """*list*, items of ``--include-plugin-directory=``"""
+    return sum([_splitShellPattern(x) for x in options.include_extra], [])
 
 
 def getShallFollowExtraFilePatterns():
-    """*list*, items of "--include-plugin-files=" """
-    return sum([_splitShellPattern(x) for x in options.recurse_extra_files], [])
+    """*list*, items of ``--include-plugin-files=``"""
+    return sum([_splitShellPattern(x) for x in options.include_extra_files], [])
 
 
 def getMustIncludeModules():
-    """*list*, items of "--include-module=" """
+    """*list*, items of ``--include-module=``"""
     return sum([_splitShellPattern(x) for x in options.include_modules], [])
 
 
 def getMustIncludePackages():
-    """*list*, items of "--include-package=" """
+    """*list*, items of ``--include-package=``"""
     return sum([_splitShellPattern(x) for x in options.include_packages], [])
 
 
 def getShallIncludePackageData():
-    """*list*, items of "--include-package-data=" """
+    """*list*, items of ``--include-package-data=``"""
     return sum([_splitShellPattern(x) for x in options.package_data], [])
 
 
 def getShallIncludeDataFiles():
-    """*list*, items of "--include-data-file=" """
+    """*list*, items of ``--include-data-file=``"""
     for data_file in options.data_files:
         if data_file.count("=") == 1:
             src, dest = data_file.split("=", 1)
@@ -579,7 +698,7 @@ def getShallIncludeDataFiles():
 
 
 def getShallIncludeDataDirs():
-    """*list*, items of "--include-data-dir=" """
+    """*list*, items of ``--include-data-dir=``"""
     for data_file in options.data_dirs:
         src, dest = data_file.split("=", 1)
 
@@ -587,42 +706,57 @@ def getShallIncludeDataDirs():
 
 
 def shallWarnImplicitRaises():
-    """*bool* = "--warn-implicit-exceptions" """
+    """:returns: bool derived from ``--warn-implicit-exceptions``"""
     return options.warn_implicit_exceptions
 
 
 def shallWarnUnusualCode():
-    """*bool* = "--warn-unusual-code" """
+    """:returns: bool derived from ``--warn-unusual-code``"""
     return options.warn_unusual_code
 
 
 def assumeYesForDownloads():
-    """*bool* = "--assume-yes-for-downloads" """
+    """:returns: bool derived from ``--assume-yes-for-downloads``"""
     return options is not None and options.assume_yes_for_downloads
 
 
 def _isDebug():
-    """*bool* = "--debug" or "--debugger" """
+    """:returns: bool derived from ``--debug`` or ``--debugger``"""
     return options is not None and (options.debug or options.debugger)
 
 
 def isPythonDebug():
-    """*bool* = "--python-debug" or "sys.flags.debug" """
+    """:returns: bool derived from ``--python-debug`` or ``sys.flags.debug``
+
+    Passed to Scons as ``python_debug`` so it can consider it when picking
+    link libraries to choose the correct variant. Also enables the define
+    ``Py_DEBUG`` for C headers. Reference counting checks and other debug
+    asserts of Python will happen in this mode.
+
+    """
     return options.python_debug or sys.flags.debug
 
 
 def isUnstripped():
-    """*bool* = "--unstripped" or "--profile" """
+    """:returns: bool derived from ``--unstripped`` or ``--profile``
+
+    A binary is called stripped when debug information is not present, an
+    unstripped when it is present. For profiling and debugging it will be
+    necessary, but it doesn*t enable debug checks like ``--debug`` does.
+
+    Passed to Scons as ``unstripped_mode`` to it can ask the linker to
+    include symbol information.
+    """
     return options.unstripped or options.profile
 
 
 def isProfile():
-    """*bool* = "--profile" """
+    """:returns: bool derived from ``--profile``"""
     return options.profile
 
 
 def shallCreateGraph():
-    """*bool* = "--graph" """
+    """:returns: bool derived from ``--graph``"""
     return options.graph
 
 
@@ -640,7 +774,7 @@ def getOutputPath(path):
 
 
 def getOutputDir():
-    """*str*, value of "--output-dir" or "." """
+    """*str*, value of ``--output-dir`` or "." """
     return options.output_dir if options.output_dir else "."
 
 
@@ -660,21 +794,37 @@ def shallOptimizeStringExec():
 
 
 def shallClearPythonPathEnvironment():
-    """*bool* = **not** "--execute-with-pythonpath" """
+    """*bool* = **not** ``--execute-with-pythonpath``"""
     return not options.keep_pythonpath
 
 
-def shallUseStaticLibPython():
-    """*bool* = "--static-libpython=yes|auto"
+_shall_use_static_lib_python = None
 
-    Notes:
-        Currently only Anaconda on non-Windows can do this and MSYS2.
-    """
+
+def _shallUseStaticLibPython():
+    # return driven, pylint: disable=too-many-return-statements
+
+    if shallMakeModule():
+        return False
 
     if options.static_libpython == "auto":
+        # Nuitka-Python is good to to static linking.
         if isNuitkaPython():
             return True
 
+        # Debian packages with Python2 are usable, Python3 will follow eventually maybe.
+        from nuitka.utils.StaticLibraries import (
+            isDebianSuitableForStaticLinking,
+        )
+
+        if (
+            python_version < 0x300
+            and isDebianPackagePython()
+            and isDebianSuitableForStaticLinking()
+            and not isPythonDebug()
+        ):
+            return True
+
         if isWin32Windows() and os.path.exists(
             os.path.join(sys.prefix, "etc/config.site")
         ):
@@ -682,23 +832,30 @@ def shallUseStaticLibPython():
 
         # For Anaconda default to trying static lib python library, which
         # normally is just not available or if it is even unusable.
-        if (
-            os.path.exists(os.path.join(sys.prefix, "conda-meta"))
-            and not isWin32Windows()
-            and not getOS() == "Darwin"
-        ):
+        if isAnacondaPython() and not isMacOS():
             return True
 
-        options.static_libpython = "no"
-
-        if getSystemStaticLibPythonPath() is not None:
-            Tracing.options_logger.info(
-                "Detected static libpython as existing, consider using '--static-libpython=yes'."
-            )
+        if isPyenvPython():
+            return True
 
     return options.static_libpython == "yes"
 
 
+def shallUseStaticLibPython():
+    """:returns: bool derived from ``--static-libpython=yes|auto`` and not module mode
+
+    Notes:
+        Currently only Anaconda on non-Windows can do this and MSYS2.
+    """
+
+    global _shall_use_static_lib_python  # singleton, pylint: disable=global-statement
+
+    if _shall_use_static_lib_python is None:
+        _shall_use_static_lib_python = _shallUseStaticLibPython()
+
+    return _shall_use_static_lib_python
+
+
 def shallTreatUninstalledPython():
     """*bool* = derived from Python installation and modes
 
@@ -716,55 +873,69 @@ def shallTreatUninstalledPython():
     if shallMakeModule() or isStandaloneMode():
         return False
 
-    # Of course only if there is a DLL.
-    if isStaticallyLinkedPython():
-        return False
-
     return isUninstalledPython()
 
 
 def isShowScons():
-    """*bool* = "--show-scons" """
+    """:returns: bool derived from ``--show-scons``"""
     return options.show_scons
 
 
 def getJobLimit():
-    """*int*, value of "--jobs" / "-j" or number of CPU kernels"""
+    """*int*, value of ``--jobs`` / "-j" or number of CPU kernels"""
+    if options.jobs is None:
+        if options.low_memory:
+            return 1
+        else:
+            return getCoreCount()
+
     return int(options.jobs)
 
 
-def isLto():
-    """*bool* = "--lto" """
+def getLtoMode():
+    """:returns: bool derived from ``--lto`` or ``--pgo``"""
     return options.lto
 
 
 def isClang():
-    """*bool* = "--clang" or enforced by platform, e.g. macOS or FreeBSD some targets."""
+    """:returns: bool derived from ``--clang`` or enforced by platform, e.g. macOS or FreeBSD some targets."""
 
     return (
         options.clang
-        or getOS() in ("Darwin", "OpenBSD")
-        or (getOS() == "FreeBSD" and getArchitecture() != "powerpc")
+        or isMacOS()
+        or isOpenBSD()
+        or (isFreeBSD() and getArchitecture() != "powerpc")
     )
 
 
 def isMingw64():
-    """*bool* = "--mingw64", available only on Windows, otherwise false"""
-    return getattr(options, "mingw64", False)
+    """:returns: bool derived from ``--mingw64``, available only on Windows, otherwise false"""
+    if isWin32Windows():
+        return options.mingw64
+    else:
+        return None
 
 
 def getMsvcVersion():
-    """*str*, value of "--msvc", available only on Windows, otherwise None"""
-    return getattr(options, "msvc", None)
+    """:returns: str derived from ``--msvc`` on Windows, otherwise None"""
+    if isWin32Windows():
+        return options.msvc_version
+    else:
+        return None
+
+
+def shallDisableCCacheUsage():
+    """:returns: bool derived from ``disable-ccache``"""
+    return options.disable_ccache
 
 
 def shallDisableConsoleWindow():
-    """*bool* = "--win-disable-console" """
-    return options.win_disable_console
+    """:returns: bool derived from ``--win-disable-console or ``--macos-disable-console``"""
+    return options.disable_console
 
 
 def _isFullCompat():
-    """*bool* = "--full-compat"
+    """:returns: bool derived from ``--full-compat``
 
     Notes:
         Code should should use "Options.is_fullcompat" instead, this
@@ -774,28 +945,32 @@ def _isFullCompat():
 
 
 def isShowProgress():
-    """*bool* = "--show-progress" """
+    """:returns: bool derived from ``--show-progress``"""
     return options is not None and options.show_progress
 
 
 def isShowMemory():
-    """*bool* = "--show-memory" """
+    """:returns: bool derived from ``--show-memory``"""
     return options is not None and options.show_memory
 
 
 def isShowInclusion():
-    """*bool* = "--show-modules" """
+    """:returns: bool derived from ``--show-modules``"""
     return options.show_inclusion
 
 
 def isRemoveBuildDir():
-    """*bool* = "--remove-output" """
+    """:returns: bool derived from ``--remove-output``"""
     return options.remove_build and not options.generate_c_only
 
 
 def getIntendedPythonArch():
-    """*str*, one of "x86", "x86_64" or None"""
-    return options.python_arch
+    """:returns: str, one of ``"x86"``, ``"x86_64"`` or ``None``
+
+    Notes: This is only available on Windows, on other platforms
+    it will be `None`
+    """
+    return options.python_arch if isWin32Windows() else None
 
 
 experimental = set()
@@ -825,7 +1000,7 @@ def disableExperimental(indication):
 
 
 def getExperimentalIndications():
-    """*tuple*, items of "--experimental=" """
+    """*tuple*, items of ``--experimental=``"""
     if hasattr(options, "experimental"):
         return options.experimental
     else:
@@ -833,23 +1008,70 @@ def getExperimentalIndications():
 
 
 def shallExplainImports():
-    """*bool* = "--explain-imports" """
+    """:returns: bool derived from ``--explain-imports``"""
     return options is not None and options.explain_imports
 
 
 def isStandaloneMode():
-    """*bool* = "--standalone" """
+    """:returns: bool derived from ``--standalone``"""
     return options.is_standalone
 
 
 def isOnefileMode():
-    """*bool* = "--onefile" """
+    """:returns: bool derived from ``--onefile``"""
     return options.is_onefile
 
 
 def isOnefileTempDirMode():
-    """*bool* = "--onefile-tempdir" """
-    return options.is_onefile_tempdir or getOS() != "Linux"
+    """:returns: bool derived from ``--onefile-tempdir`` and OS
+
+    Notes:
+        On all but Linux, using a bootstrap binary that does unpack is mandatory,
+        but on Linux, the AppImage tool is used by default, this enforces using
+        a bootstrap binary there too.
+    """
+    return not isLinux() or options.is_onefile_tempdir
+
+
+def isPgoMode():
+    """:returns: bool derived from ``--pgo``"""
+    return options.is_c_pgo
+
+
+def isPythonPgoMode():
+    """:returns: bool derived from ``--pgo-python``"""
+    return options.is_python_pgo
+
+
+def getPythonPgoInput():
+    """:returns: str derived from ``--pgo-python-input``"""
+    return options.python_pgo_input
+
+
+def shallCreatePgoInput():
+    return isPythonPgoMode() and getPythonPgoInput() is None
+
+
+def getPgoArgs():
+    """*list* = ``--pgo-args``"""
+    return shlex.split(options.pgo_args)
+
+
+def getPgoExecutable():
+    """*str* = ``--pgo-args``"""
+
+    if options.pgo_executable and os.path.exists(options.pgo_executable):
+        if not os.path.isabs(options.pgo_executable):
+            options.pgo_executable = os.path.normcase(
+                os.path.join(".", options.pgo_executable)
+            )
+
+    return options.pgo_executable
+
+
+def getPythonPgoUnseenModulePolicy():
+    """*str* = ``--python-pgo-unused-module-policy``"""
+    return options.python_pgo_policy_unused_module
 
 
 def getOnefileTempDirSpec(use_default):
@@ -863,12 +1085,12 @@ def getOnefileTempDirSpec(use_default):
 
 
 def getIconPaths():
-    """*list of str*, values of "--windows-icon-from-ico" and "--linux-onefile-icon"""
+    """*list of str*, values of ``--windows-icon-from-ico`` and ``--linux-onefile-icon``"""
 
     result = options.icon_path
 
     # Check if Linux icon requirement is met.
-    if getOS() == "Linux" and not result and isOnefileMode():
+    if isLinux() and not result and isOnefileMode():
         default_icons = (
             "/usr/share/pixmaps/python%s.xpm" % python_version_str,
             "/usr/share/pixmaps/python%s.xpm" % sys.version_info[0],
@@ -882,7 +1104,7 @@ def getIconPaths():
         else:
             Tracing.options_logger.sysexit(
                 """\
-Error, the non of the default icons '%s' exist, making --linux-onefile-icon required."""
+Error, none of the default icons '%s' exist, making '--linux-onefile-icon' required."""
                 % ", ".join(default_icons)
             )
 
@@ -890,17 +1112,17 @@ Error, the non of the default icons '%s' exist, making --linux-onefile-icon requ
 
 
 def getWindowsIconExecutablePath():
-    """*str* or *None* if not given, value of "--windows-icon-from-exe" """
+    """*str* or *None* if not given, value of ``--windows-icon-from-exe``"""
     return options.icon_exe_path
 
 
 def shallAskForWindowsAdminRights():
-    """*bool*, value of "--windows-uac-admin" or --windows-uac-uiaccess"""
+    """*bool*, value of ``--windows-uac-admin`` or ``--windows-uac-uiaccess``"""
     return options.windows_uac_admin
 
 
 def shallAskForWindowsUIAccessRights():
-    """*bool*, value of "--windows-uac-uiaccess" """
+    """*bool*, value of ``--windows-uac-uiaccess``"""
     return options.windows_uac_uiaccess
 
 
@@ -941,15 +1163,20 @@ def _parseWindowsVersionNumber(value):
 
 
 def getWindowsProductVersion():
-    """*tuple of 4 ints* or None --windows-product-version"""
+    """:returns: tuple of 4 ints or None, derived from ``--windows-product-version``"""
     return _parseWindowsVersionNumber(options.windows_product_version)
 
 
 def getWindowsFileVersion():
-    """*tuple of 4 ints* or None --windows-file-version"""
+    """:returns tuple of 4 ints or None, derived from ``--windows-file-version``"""
     return _parseWindowsVersionNumber(options.windows_file_version)
 
 
+def getWindowsSplashScreen():
+    """:returns: bool derived from ``--onefile-windows-splash-screen-image``"""
+    return options.splash_screen_image
+
+
 def getWindowsCompanyName():
     """*str* name of the company to use"""
     return options.windows_company_name
@@ -960,11 +1187,31 @@ def getWindowsProductName():
     return options.windows_product_name
 
 
+def shallCreateAppBundle():
+    """*bool* shall create an application bundle"""
+    return options.macos_create_bundle
+
+
+def getMacOSAppName():
+    """*str* name of the app to use bundle"""
+    return options.macos_app_name
+
+
+def getMacOSSignedAppName():
+    """*str* name of the app to use during signing"""
+    return options.macos_signed_app_name
+
+
+def getMacOSAppVersion():
+    """*str* version of the app to use for bundle"""
+    return options.macos_app_version
+
+
 _python_flags = None
 
 
-def getPythonFlags():
-    """*list*, value of "--python-flag" """
+def _getPythonFlags():
+    """*list*, values of ``--python-flag``"""
     # singleton, pylint: disable=global-statement
     global _python_flags
 
@@ -975,7 +1222,12 @@ def getPythonFlags():
             for part in parts.split(","):
                 if part in ("-S", "nosite", "no_site"):
                     _python_flags.add("no_site")
-                elif part in ("static_hashes", "norandomization", "no_randomization"):
+                elif part in (
+                    "-R",
+                    "static_hashes",
+                    "norandomization",
+                    "no_randomization",
+                ):
                     _python_flags.add("no_randomization")
                 elif part in ("-v", "trace_imports", "trace_import"):
                     _python_flags.add("trace_imports")
@@ -999,25 +1251,43 @@ def getPythonFlags():
 def hasPythonFlagNoSite():
     """*bool* = "no_site" in python flags given"""
 
-    return "no_site" in getPythonFlags()
+    return "no_site" in _getPythonFlags()
 
 
 def hasPythonFlagNoAnnotations():
     """*bool* = "no_annotations" in python flags given"""
 
-    return "no_annotations" in getPythonFlags()
+    return "no_annotations" in _getPythonFlags()
 
 
 def hasPythonFlagNoAsserts():
     """*bool* = "no_asserts" in python flags given"""
 
-    return "no_asserts" in getPythonFlags()
+    return "no_asserts" in _getPythonFlags()
 
 
 def hasPythonFlagNoDocstrings():
     """*bool* = "no_docstrings" in python flags given"""
 
-    return "no_docstrings" in getPythonFlags()
+    return "no_docstrings" in _getPythonFlags()
+
+
+def hasPythonFlagNoWarnings():
+    """*bool* = "no_docstrings" in python flags given"""
+
+    return "no_warnings" in _getPythonFlags()
+
+
+def hasPythonFlagTraceImports():
+    """*bool* = "trace_imports", "-v" in python flags given"""
+
+    return "trace_imports" in _getPythonFlags()
+
+
+def hasPythonFlagNoRandomization():
+    """*bool* = "no_randomization", "-R", "static_hashes" in python flags given"""
+
+    return "no_randomization" in _getPythonFlags()
 
 
 def shallFreezeAllStdlib():
@@ -1026,19 +1296,19 @@ def shallFreezeAllStdlib():
 
 
 def getWindowsDependencyTool():
-    """*str*, value of "--windows-dependency-tool=" """
+    """*str*, value of ``--windows-dependency-tool=``"""
     return options.dependency_tool
 
 
 def shallNotUseDependsExeCachedResults():
-    """*bool* = "--disable-dll-dependency-cache" or "--force-dll-dependency-cache-update" """
+    """:returns: bool derived from ``--disable-dll-dependency-cache`` or ``--force-dll-dependency-cache-update``"""
     return shallNotStoreDependsExeCachedResults() or getattr(
         options, "update_dependency_cache", False
     )
 
 
 def shallNotStoreDependsExeCachedResults():
-    """*bool* = "--disable-dll-dependency-cache" """
+    """:returns: bool derived from ``--disable-dll-dependency-cache``"""
     return getattr(options, "no_dependency_cache", False)
 
 
@@ -1093,7 +1363,7 @@ def getPluginsDisabled():
 
 
 def getUserPlugins():
-    """*tuple*, items user provided of "--user-plugin=" """
+    """*tuple*, items user provided of ``--user-plugin=``"""
     if not options:
         return ()
 
@@ -1101,18 +1371,22 @@ def getUserPlugins():
 
 
 def shallDetectMissingPlugins():
-    """*bool* = **not** "--plugin-no-detection" """
+    """*bool* = **not** ``--plugin-no-detection``"""
     return options is not None and options.detect_missing_plugins
 
 
 def getPythonPathForScons():
-    """*str*, value of "--python-for-scons" """
+    """*str*, value of ``--python-for-scons``"""
     return options.python_scons
 
 
 def shallCompileWithoutBuildDirectory():
     """*bool* currently hard coded, not when using debugger.
 
+    When this is used, compilation is executed in a fashion that it runs
+    inside the build folder, hiding it, attempting to make results more
+    reproducible across builds of different programs.
+
     TODO: Make this not hardcoded, but possible to disable via an
     options.
     """
@@ -1137,3 +1411,13 @@ def getForcedStdoutPath():
 def getForcedStderrPath():
     """*str* force program stderr output into that filename"""
     return options.force_stderr_spec
+
+
+def shallPersistModifications():
+    """*bool* write plugin source changes to disk"""
+    return options is not None and options.persist_source_changes
+
+
+def isLowMemory():
+    """*bool* low memory usage requested"""
+    return options.low_memory
