@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,8 +17,8 @@
 #
 """ Reformulation of with statements.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
@@ -28,8 +28,8 @@ from nuitka.nodes.AssignNodes import (
     StatementReleaseVariable,
 )
 from nuitka.nodes.AttributeNodes import (
-    ExpressionAttributeLookup,
     ExpressionAttributeLookupSpecial,
+    makeExpressionAttributeLookup,
 )
 from nuitka.nodes.CallNodes import (
     ExpressionCallEmpty,
@@ -70,10 +70,10 @@ from .TreeHelpers import (
 
 
 def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref):
-    # Many details, pylint: disable=too-many-locals
+    # Many details, pylint: disable=too-many-branches,too-many-locals
     with_source = buildNode(provider, context_expr, source_ref)
 
-    if python_version < 380 and Options.is_fullcompat:
+    if python_version < 0x380 and Options.is_fullcompat:
         source_ref = with_source.getCompatibleSourceReference()
 
     temp_scope = provider.allocateTempScope("with")
@@ -116,7 +116,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
         while deepest.getVisitableNodes():
             deepest = deepest.getVisitableNodes()[-1]
 
-        if python_version < 370:
+        if python_version < 0x370:
             body_lineno = deepest.getCompatibleSourceReference().getLineNumber()
         else:
             body_lineno = deepest.getSourceReference().getLineNumber()
@@ -127,13 +127,13 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
 
     # The "__enter__" and "__exit__" were normal attribute lookups under
     # CPython2.6, but that changed with CPython2.7.
-    if python_version < 270:
-        attribute_lookup_class = ExpressionAttributeLookup
+    if python_version < 0x270:
+        attribute_lookup_maker = makeExpressionAttributeLookup
     else:
-        attribute_lookup_class = ExpressionAttributeLookupSpecial
+        attribute_lookup_maker = ExpressionAttributeLookupSpecial
 
     enter_value = ExpressionCallEmpty(
-        called=attribute_lookup_class(
+        called=attribute_lookup_maker(
             expression=ExpressionTempVariableRef(
                 variable=tmp_source_variable, source_ref=source_ref
             ),
@@ -168,12 +168,6 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
 
     # For "async with", await the entered value and exit value must be awaited.
     if not sync:
-        enter_value = ExpressionYieldFromWaitable(
-            expression=ExpressionAsyncWaitEnter(
-                expression=enter_value, source_ref=source_ref
-            ),
-            source_ref=source_ref,
-        )
         exit_value_exception = ExpressionYieldFromWaitable(
             expression=ExpressionAsyncWaitExit(
                 expression=exit_value_exception, source_ref=source_ref
@@ -187,36 +181,69 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
             source_ref=source_ref,
         )
 
+    # First assign the with context to a temporary variable.
     statements = [
-        # First assign the with context to a temporary variable.
         StatementAssignmentVariable(
             variable=tmp_source_variable, source=with_source, source_ref=source_ref
         )
     ]
 
-    attribute_assignments = [
-        # Next, assign "__enter__" and "__exit__" attributes to temporary
-        # variables.
-        StatementAssignmentVariable(
-            variable=tmp_exit_variable,
-            source=attribute_lookup_class(
-                expression=ExpressionTempVariableRef(
-                    variable=tmp_source_variable, source_ref=source_ref
+    # Before 3.9, __aenter__ is immediately awaited, after we first do __aexit__ lookup.
+    if not sync and python_version < 0x390:
+        enter_value = ExpressionYieldFromWaitable(
+            expression=ExpressionAsyncWaitEnter(
+                expression=enter_value, source_ref=source_ref
+            ),
+            source_ref=source_ref,
+        )
+
+    attribute_enter_assignment = StatementAssignmentVariable(
+        variable=tmp_enter_variable, source=enter_value, source_ref=source_ref
+    )
+
+    attribute_exit_assignment = StatementAssignmentVariable(
+        variable=tmp_exit_variable,
+        source=attribute_lookup_maker(
+            expression=ExpressionTempVariableRef(
+                variable=tmp_source_variable, source_ref=source_ref
+            ),
+            attribute_name="__exit__" if sync else "__aexit__",
+            source_ref=source_ref,
+        ),
+        source_ref=source_ref,
+    )
+
+    # Next, assign "__enter__" and "__exit__" attributes to temporary variables, and
+    # depending on Python versions switch the order of these lookups and the order of
+    # awaiting enter.
+    # Normal "with" statements are enter, exit ordered after 3.6, and "async with"
+    # are since 3.9, and since 3.9 the enter is not awaited, until an exit is present.
+    if python_version >= 0x390 and not sync:
+        enter_await_statement = StatementAssignmentVariable(
+            variable=tmp_enter_variable,
+            source=ExpressionYieldFromWaitable(
+                expression=ExpressionAsyncWaitEnter(
+                    expression=ExpressionTempVariableRef(
+                        variable=tmp_enter_variable, source_ref=source_ref
+                    ),
+                    source_ref=source_ref,
                 ),
-                attribute_name="__exit__" if sync else "__aexit__",
                 source_ref=source_ref,
             ),
             source_ref=source_ref,
-        ),
-        StatementAssignmentVariable(
-            variable=tmp_enter_variable, source=enter_value, source_ref=source_ref
-        ),
-    ]
+        )
 
-    if python_version >= 360 and sync:
-        attribute_assignments.reverse()
+        attribute_assignments = (
+            attribute_enter_assignment,
+            attribute_exit_assignment,
+            enter_await_statement,
+        )
+    elif python_version >= 0x360 and sync:
+        attribute_assignments = (attribute_enter_assignment, attribute_exit_assignment)
+    else:
+        attribute_assignments = (attribute_exit_assignment, attribute_enter_assignment)
 
-    statements += attribute_assignments
+    statements.extend(attribute_assignments)
 
     statements.append(
         StatementAssignmentVariable(
@@ -255,7 +282,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
                     ),
                     source_ref=source_ref,
                 ),
-                public_exc=python_version >= 270,
+                public_exc=python_version >= 0x270,
                 source_ref=source_ref,
             ),
             final=makeStatementConditional(
