@@ -42,6 +42,7 @@ sys.path.insert(
 from nuitka.tools.testing.Common import (
     compareWithCPython,
     createSearchMode,
+    getMainProgramFilename,
     my_print,
     reportSkip,
     setup,
@@ -49,23 +50,11 @@ from nuitka.tools.testing.Common import (
 )
 
 
-def getMainProgramFilename(filename):
-    for filename_main in os.listdir(filename):
-        if filename_main.endswith(("Main.py", "Main")):
-            return filename_main
-
-    sys.exit(
-        """\
-Error, no file ends with 'Main.py' or 'Main' in %s, incomplete test case."""
-        % (filename)
-    )
-
-
 def main():
     # Complex stuff, even more should become common code though.
     # pylint: disable=too-many-branches,too-many-statements
 
-    python_version = setup(needs_io_encoding=True)
+    python_version = setup(suite="programs", needs_io_encoding=True)
 
     search_mode = createSearchMode()
 
@@ -127,15 +116,15 @@ def main():
 
         extra_flags.append("remove_output")
 
-        extra_flags.append("recurse_all")
+        extra_flags.append("--follow-imports")
 
         # Use the original __file__ value, at least one case warns about things
         # with filename included, but for pkgutil iteration, make sure we do not
         # see original Python dirs.
         if filename != "pkgutil_itermodules":
-            extra_flags.append("original_file")
+            extra_flags.append("--file-reference-choice=original")
         else:
-            extra_flags.append("runtime_file")
+            extra_flags.append("--file-reference-choice=runtime")
 
         # Cannot include the files with syntax errors, these would then become
         # ImportError, but that's not the test. In all other cases, use two
@@ -166,10 +155,8 @@ def main():
 
             extra_flags.append("ignore_warnings")
         elif filename == "multiprocessing_using":
-            if os.name == "nt":
-                extra_flags.append("plugin_enable:multiprocessing")
-
-            elif sys.platform == "darwin" and python_version >= (3, 8):
+            # TODO: Still true?
+            if sys.platform == "darwin" and python_version >= (3, 8):
                 reportSkip("Hangs for unknown reasons", ".", filename)
                 continue
         else:
